/*
 * XF-SPF, basic but fully function implementation of SPF checker to be
 * used with qmail-smtpd's SMTPEXTFORK patch. Please see README for details.
 */
#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <arpa/nameser.h> /* DNS struct */
#include "xf-spf.h"
#include "syslog.h"
#include "envelope_vrf.h"

#include <spf2/spf.h>

/*
 *  Function to add timestamp to logs
 */
char * tlg2()
{
  static char buff[23];
  time_t tm;
  tm = time(NULL);
  ctime_r(&tm,buff);
  sprintf(buff,"%.24s",buff);
 return(buff);
}

int check_spf()
{
	int	res = 1;
	int	i;
	int	xfspf_dbg = 0;

	char *remote_ip = NULL;
	char *remote_sender = NULL;
	char *heloehlo = NULL;
	char *rcptto = NULL;

	const char *envp;
	int sfspfbehavior = 0;

	SPF_server_t		*spf_server = NULL;
	SPF_request_t		*spf_request = NULL;
	SPF_response_t		*spf_myres = NULL;
	SPF_response_t		*spf_myres2 = NULL; // para DNS secundario

	FILE *sflog = NULL;

int error(int res)
{
	if (spf_request)
		SPF_request_free(spf_request);
	if (spf_server)
		SPF_server_free(spf_server);
	return(res);
}


	// getenv retorna char * entao, convertemos ponteiro pra inteiro e atribuimos
	if (!(envp = getenv("SFSPFBEHAVIOR"))) {
		loga("SFSPFBEHAVIOR enviroment var is not defined");
		return 1;
	} else
		sfspfbehavior = atoi(envp);

	// if sfspfbehavior is not 1-6, no more checks, let it pass
	if ((sfspfbehavior < 1) || (sfspfbehavior > 6)) return 1;

	if (!(remote_ip = getenv("TCPREMOTEIP"))) exit(NOTCLI);
	if (!(rcptto = getenv("REMOTERCPT"))) exit(NOTCLI);
	if (!(heloehlo = getenv("REMOTEHELO"))) exit(NOTCLI);
	if (!(remote_sender = getenv("REMOTEMF"))) remote_sender = "postmaster";

	spf_server = SPF_server_new(SPF_DNS_CACHE, xfspf_dbg);

	if (spf_server == NULL) {
		loga("SPF lib failed on spf_server creation");
		error(XFSPFERR);
	}

	spf_request = SPF_request_new(spf_server);

	if ( SPF_request_set_ipv4_str( spf_request, remote_ip ) ) {
		loga("SPF lib identified IP address as invalid");
		error(XFSPFERR);
	}

	if (heloehlo == NULL) {
		if (remote_sender != NULL) {
			if (strstr(remote_sender, "@") != NULL) {
				heloehlo = strdup(strstr(remote_sender, "@") + 1);

				if ( SPF_request_set_helo_dom( spf_request, heloehlo ) ) {
					loga("SPF lib identified HELO domain as invalid");
					error(XFSPFERR);
				}
			}
		}
	} else {
		if ( SPF_request_set_helo_dom( spf_request, heloehlo ) ) {
			loga("SPF lib identified HELO domain as invalid");
			error(XFSPFERR);
		}
	}

	if ( SPF_request_set_env_from( spf_request, remote_sender ) ) {
		loga("SPF lib identified envelope FROM address as invalid");
		error(XFSPFERR);
	}

	SPF_request_query_mailfrom(spf_request, &spf_myres);

	if (SPF_response_result(spf_myres) != SPF_RESULT_PASS) {
		SPF_request_query_rcptto(spf_request, &spf_myres2, rcptto);
		if (SPF_response_result(spf_myres2) == SPF_RESULT_PASS) {
		}
	}

	// Vamos liberar a memoria
	SPF_response_free(spf_myres);
	if (spf_myres2) SPF_response_free(spf_myres2);

	if (sfspfbehavior > 0)
		if (envp = SPF_response_get_received_spf(spf_myres))
			loga(envp);
#if LOGNSPFDOM == 1
		else
			loga("Received-SPF: none (domain does not have SPF record)");
#endif

	switch (res = SPF_response_result(spf_myres)) {
	  case SPF_RESULT_PASS:
	    sflog = fopen(SFLOGFILE,"a");
	      fprintf(sflog,"%s softfail: %s -> %s (%s) SPF Accept: pass (behavior = %i)\n", tlg2(), mf_addr, sf_rcpt_l[0].mail, mf_fqdnaddr, sfspfbehavior);
	    fclose(sflog);
	    res=0;
	  break;
	  default:
	    return 1;
	    res=1;
	  break;
	}

#ifdef DEBUGTLOG
 sflog = fopen(SFLOGFILE,"a");
   fprintf(sflog,"%s softfail DEBUG : %s -> %s (%s) sfspfbehavior: %i\n", tlg2(), mf_addr, sf_rcpt_l[0].mail, mf_fqdnaddr,sfspfbehavior);
   
 fclose(sflog);
#endif
return(res);
}
